{-# LANGUAGE DeriveGeneric #-}
{-# LANGUAGE DuplicateRecordFields #-}

module Types.WebSocketMessages.WebSocketMessages
  ( WebSocketMessage (..),
    SetClientInfo (..),
    JoinRoom (..),
    AllChatMessageIncoming (..),
    AllChatMessageOutgoing (..),
  )
where

import ClassyPrelude
import Data.Aeson
  ( FromJSON (parseJSON),
    Options (sumEncoding),
    SumEncoding (..),
    ToJSON,
    defaultOptions,
    genericParseJSON,
  )
import Data.Time.Clock.POSIX (POSIXTime)
import Data.UUID (UUID)
import Types.User (User)

data WebSocketMessage = ClientInfoMessage SetClientInfo | JoinRoomMessage JoinRoom | AllChatMessageIncomingMessage AllChatMessageIncoming
  deriving (Generic)

instance FromJSON WebSocketMessage where
  parseJSON = genericParseJSON defaultOptions {sumEncoding = UntaggedValue}

data SetClientInfo where
  SetClientInfo :: {displayName :: Text} -> SetClientInfo
  deriving (Generic, Show)

instance FromJSON SetClientInfo

data JoinRoom where
  JoinRoom :: {roomName :: Text} -> JoinRoom
  deriving (Generic, Show)

instance FromJSON JoinRoom

data AllChatMessageIncoming where
  AllChatMessageIncoming ::
    {content :: Text} ->
    AllChatMessageIncoming
  deriving (Generic, Show)

instance FromJSON AllChatMessageIncoming

data AllChatMessageOutgoing = AllChatMessageOutgoing
  { content :: Text,
    sender :: User,
    uuid :: UUID,
    timestamp :: POSIXTime
  }
  deriving (Generic, Show)

instance ToJSON AllChatMessageOutgoing
