{-# LANGUAGE FlexibleInstances #-}

module WebSocket.MonadWebSocketSession
  ( MonadWebSocketSession (..),
    MonadWebSocketSessionInit (..),
  )
where

import BroadcastUserData (MonadBroadcast, broadcastUserData)
import ClassyPrelude
import Data.Aeson
  ( FromJSON,
    eitherDecodeStrict,
  )
import Data.UUID (UUID)
import State.ConnectedClientsState
  ( MonadConnectedClientsModify,
    MonadConnectedClientsRead (getConnctedClients),
    removeWSClient,
  )
import State.RoomDataState (MonadRoomDataStateRead)
import Types.ConnectionState (Client (..))
import Types.WebSocketMessages.WebSocketMessages (SetClientInfo (..))
import WebSocket.Messages
import WebSocket.WSReaderTApp

class MonadConnectedClientsRead m => MonadWebSocketSession m where
  getTypedWSMessage :: FromJSON a => m a
  getSesssionId :: m UUID
  getClient :: m (Maybe Client)

instance MonadWebSocketSession (WSApp WSEnv) where
  getTypedWSMessage = do
    msg <- getMessage
    case eitherDecodeStrict $ encodeUtf8 msg of
      Right a -> return a
      Left err -> do
        sendMessage $ "Bad message: " <> pack err
        getTypedWSMessage
  getSesssionId = getClientId <$> ask
  getClient = do
    id' <- getSesssionId
    find ((== id') . uuid) <$> getConnctedClients

class (Monad m) => MonadWebSocketSessionInit m where
  newClient :: SetClientInfo -> m Client
  withCleanUp :: m () -> m ()

instance MonadWebSocketSessionInit (WSApp WSEnv) where
  newClient = newClientGeneric
  withCleanUp = flip finally disconnectWsClient

newClientGeneric ::
  ( MonadIO m,
    HasWSEnv env,
    MonadReader env m
  ) =>
  SetClientInfo ->
  m Client
newClientGeneric clientInfo = do
  env <- ask
  return $ Client {uuid = getClientId env, name = displayName clientInfo, conn = getConn env, joinedRoom = False}

disconnectWsClient ::
  ( MonadConnectedClientsModify m,
    MonadRoomDataStateRead m,
    MonadBroadcast m,
    MonadWebSocketSession m
  ) =>
  m ()
disconnectWsClient = do
  clientId <- getSesssionId
  removeWSClient clientId
  broadcastUserData
